@@ -5,7 +5,7 @@ use warnings;
 package Mite::Miteception;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.007006';
+our $VERSION   = '0.008000';
 
 use Import::Into;
 use Mite::Shim ();
@@ -21,7 +21,7 @@ sub import {
 		require Mite::Project;
 		Mite::Project->default->inject_mite_functions(
 			 package     => $caller,
-			 file        => $file,
+			 file        => $ENV{MITE_COMPILE_SELF},
 			 arg         => \%arg,
 			 kind        => ( $arg{'-role'} ? 'role' : 'class' ),
 			 shim        => 'Mite::Shim',
