@@ -6,7 +6,7 @@ package Mite::Signature;
 use Mite::Miteception -all;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.007006';
+our $VERSION   = '0.008000';
 
 has class =>
   is            => ro,
@@ -43,7 +43,7 @@ has method =>
 has head =>
   is            => lazy,
   isa           => ArrayRef | Int,
-  builder       => sub { shift->method ? [ Defined ] : [] };
+  builder       => sub { shift->method ? [ Defined, { name => 'invocant' } ] : [] };
 
 has tail =>
   is            => ro,
